/**
 * \file formula.h
 * \author Léo Perrin
 * \date 2011/11/22
 */

#ifndef _FORMULA_H
#define _FORMULA_H_

#include <vector>
#include <string>
#include <fstream>

#include "clause.h"
#include "variableset.h"

/**
 * \class Formula
 * \brief An instance implements a complete CNF formula using several
 * Clause instances.
 *
 * A CNF formula is the conjunction of an arbitrary number of
 * disjunctive clauses. These are represented by Clause instances stored
 * in a vector. Note that the codes of the literals of the clauses are
 * given by a VariableSet instance connected to this one.
 */
class Formula
{
	private:
		/**
		 * \brief The set of the variables.
		 * 
		 * It "lives" independantly of the formula so it is a pointer
		 * to it.
		 */
		VariableSet * varSet;

		/**
		 * \brief Stores the CNF clauses of this Formula as Clause
		 * instances.
		 */
		std::vector<Clause*> cnfs;

		/**
		 * \brief The clause currently being modified. It is
		 * modified by the newClause(), addLit() and
		 * pushClause().
		 */
		Clause * bufferClause;

	public:
		/**
		 * \brief Creates an instance of Formula which will use the
		 * VariableSet of the input.
		 *
		 * \param S A pointer to the VariableSet to link to this Formula.
		*/
		Formula(VariableSet*);

		/**
		 * \brief Reinitialise the bufferClause attribute to
		 * an emtpy one.
		 */
		void newClause();

		/**
		 * \brief Adds a new variable to the bufferClause
		 * attribute, negated if sign==false.
                 * 
                 * \param s The subset in which to take the variable
                 * \param sign False if the literal is negated, true otherwise.
                 * \param i First coordinate.
                 * \param j Second coordinate.
		 */
		void addLit(unsigned int, bool, unsigned int, unsigned int);

		/**
		 * \brief Adds a new variable to the bufferClause
		 * attribute, negated if sign==false
                 *
                 * \param s The subset in which to take the variable
                 * \param sign False if the literal is negated, true otherwise.
                 * \param i First coordinate.
                 * \param j Second coordinate.
                 * \param k Third coordinate.
		 */
		void addLit(unsigned int, bool, unsigned int, unsigned int, unsigned int);

		/**
		 * \brief Use this method to put the current clause in
		 * the formula modeled by this instance.
		 */
		void pushClause();

		/**
		 * \brief Applies the restriction corresponding to the partial
		 * assignment given as input to all the clauses of this formula.
		 *
		 * It uses the restriction method of the Clause object on each
		 * one of those in cnfs and removes the satisfied Clauses from
		 * cnfs.
		 */
		void restriction(std::vector<int>);


		/**
		 * \brief Creates the DIMACS formated file corresponding to this
		 * formula at the place specified by the first input.
		 *
		 * For more info on the DIMACS encoding, see <a href="www.csc.
		 * kth.se/~jakobn/teaching/proofcplx11/minisat.php">this</a>. A
		 * commentary will be added in the beginning of the file.
		 *
		 * \param output A pointer to the stream where the file must be
		 * written.
		 * \param comment The commentary to add on the first line.
		*/
		void toFile(std::ostream *,std::string);
};

#endif
